﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Reflection;
using System.Diagnostics;



namespace FLY.IntegratedControl.Common
{
    public class ICData : INotifyPropertyChanged
    {
        public ICData()
        {
        }

        #region 辅助代码生成
        /// <summary>
        /// 进风指示
        /// </summary>
        public bool IsInletAirOn { get; set; }

        /// <summary>
        /// 进风运行频率(Hz)(自动调节)
        /// </summary>
        public float InletAirFreq { get; set; }

        /// <summary>
        /// 进风设定频率(Hz)
        /// </summary>
        public float InletAirFreqSet { get; set; }

        /// <summary>
        /// 出风指示
        /// </summary>
        public bool IsOutletAirOn { get; set; }

        /// <summary>
        /// 出风运行频率(Hz)
        /// </summary>
        public float OutletAirFreq { get; set; }

        /// <summary>
        /// 出风设定频率(Hz)
        /// </summary>
        public float OutletAirFreqSet { get; set; }

        /// <summary>
        /// 出风设定频率后通知
        /// </summary>
        public bool IsOutletAirFreqChanged { get; set; }

        /// <summary>
        /// 折径显示(mm)
        /// </summary>
        public float FilmWidth { get; set; }

        /// <summary>
        /// 折径设定(mm)
        /// </summary>
        public float FilmWidthSet { get; set; }

        /// <summary>
        /// 折径设定后通知
        /// </summary>
        public bool IsFilmWidthChanged { get; set; }

        /// <summary>
        /// 误差校正
        /// </summary>
        public float ErrorCorrection { get; set; }

        /// <summary>
        /// IBC自动
        /// </summary>
        public bool IsIBCAuto { get; set; }

        /// <summary>
        /// 外冷指示
        /// </summary>
        public bool IsExCoolOn { get; set; }

        /// <summary>
        /// 外冷运行频率(Hz)
        /// </summary>
        public float IsExCoolFreq { get; set; }

        /// <summary>
        /// 外冷设定频率(Hz)
        /// </summary>
        public float IsExCoolFreqSet { get; set; }

        /// <summary>
        /// K1
        /// </summary>
        public float K1 { get; set; }

        /// <summary>
        /// K2
        /// </summary>
        public float K2 { get; set; }

        /// <summary>
        /// K3
        /// </summary>
        public float K3 { get; set; }

        /// <summary>
        /// L
        /// </summary>
        public float L { get; set; }

        /// <summary>
        /// 膜泡校正A
        /// </summary>
        public float BubbleCorrA { get; set; }

        /// <summary>
        /// 膜泡校正B
        /// </summary>
        public float BubbleCorrB { get; set; }

        /// <summary>
        /// 膜泡校正C
        /// </summary>
        public float BubbleCorrC { get; set; }

        /// <summary>
        /// 膜泡校正R
        /// </summary>
        public float BubbleCorrR { get; set; }

        /// <summary>
        /// 采样周期
        /// </summary>
        public float SampleInterval { get; set; }

        /// <summary>
        /// 区间1设置
        /// </summary>
        public float Area1Set { get; set; }

        /// <summary>
        /// 区间1P
        /// </summary>
        public float Area1P { get; set; }

        /// <summary>
        /// 区间1I
        /// </summary>
        public float Area1I { get; set; }

        /// <summary>
        /// 区间1D
        /// </summary>
        public float Area1D { get; set; }

        /// <summary>
        /// 区间1DI
        /// </summary>
        public float Area1DI { get; set; }

        /// <summary>
        /// 调节系数
        /// </summary>
        public float AdjustFactor { get; set; }

        /// <summary>
        /// PID结果
        /// </summary>
        public float PIDResult { get; set; }

        /// <summary>
        /// PID调节
        /// </summary>
        public float PIDAdjust { get; set; }

        /// <summary>
        /// 进风速度
        /// </summary>
        public float InletAirVelocity { get; set; }

        /// <summary>
        /// 出风速度
        /// </summary>
        public float OutletAirVelocity { get; set; }

        /// <summary>
        /// 速度系数
        /// </summary>
        public float VelocityFactor { get; set; }

        /// <summary>
        /// 牵引直径
        /// </summary>
        public float TractionDiameter { get; set; }

        /// <summary>
        /// 牵引速比
        /// </summary>
        public float TractionVRatio { get; set; }

        /// <summary>
        /// 牵引速度
        /// </summary>
        public float TractionVelocity { get; set; }

        /// <summary>
        /// 区间2设置
        /// </summary>
        public float Area2Set { get; set; }

        /// <summary>
        /// 区间2P
        /// </summary>
        public float Area2P { get; set; }

        /// <summary>
        /// 区间2I
        /// </summary>
        public float Area2I { get; set; }

        /// <summary>
        /// 区间2D
        /// </summary>
        public float Area2D { get; set; }

        /// <summary>
        /// 传感器1盲区
        /// </summary>
        public float Sensor1DeadZone { get; set; }

        /// <summary>
        /// 传感器2盲区
        /// </summary>
        public float Sensor2DeadZone { get; set; }

        /// <summary>
        /// 传感器3盲区
        /// </summary>
        public float Sensor3DeadZone { get; set; }

        /// <summary>
        /// 传感器1系数
        /// </summary>
        public float Sensor1Factor { get; set; }

        /// <summary>
        /// 传感器2系数
        /// </summary>
        public float Sensor2Factor { get; set; }

        /// <summary>
        /// 传感器3系数
        /// </summary>
        public float Sensor3Factor { get; set; }

        /// <summary>
        /// 外冷额定频率
        /// </summary>
        public float ExCoolDefaultFreq { get; set; }

        /// <summary>
        /// 出风额定频率
        /// </summary>
        public float OutletAirDefaultFreq { get; set; }

        /// <summary>
        /// 进风额定频率
        /// </summary>
        public float InletAirDefaultFreq { get; set; }

        /// <summary>
        /// 牵引额定频率
        /// </summary>
        public float TractionDefaultFreq { get; set; }

        /// <summary>
        /// 旋转额定频率
        /// </summary>
        public float RotationDefaultFreq { get; set; }

        /// <summary>
        /// 螺杆速度
        /// </summary>
        public float ScrewVelocity { get; set; }

        /// <summary>
        /// 螺杆速度设置
        /// </summary>
        public float ScrewVelocitySet { get; set; }

        /// <summary>
        /// 主牵引运行指示
        /// </summary>
        public bool IsTractionOn { get; set; }

        /// <summary>
        /// 主牵引开
        /// </summary>
        public bool TractionOn { get; set; }

        /// <summary>
        /// 气垫辊风机运行指示
        /// </summary>
        public bool IsAirCushionOn { get; set; }

        /// <summary>
        /// 气垫辊风机开
        /// </summary>
        public bool AirCushionOn { get; set; }

        /// <summary>
        /// 旋转塔正转指示
        /// </summary>
        public bool IsRotaryForw { get; set; }

        /// <summary>
        /// 旋转塔反转指示
        /// </summary>
        public bool IsRotaryBackw { get; set; }

        /// <summary>
        /// 旋转塔正转
        /// </summary>
        public bool RotaryForw { get; set; }

        /// <summary>
        /// 旋转塔反转
        /// </summary>
        public bool RotaryBackw { get; set; }

        /// <summary>
        /// 旋转塔正向机械限位
        /// </summary>
        public bool IsRotaryForwLimit { get; set; }

        /// <summary>
        /// 旋转塔正转换向限位
        /// </summary>
        public bool IsRotaryForwTurn { get; set; }

        /// <summary>
        /// 旋转塔复位原点限位
        /// </summary>
        public bool IsRotaryOrgSign { get; set; }

        /// <summary>
        /// 旋转塔反转换向限位
        /// </summary>
        public bool IsRotaryBackwTurn { get; set; }

        /// <summary>
        /// 旋转塔反向机械限位
        /// </summary>
        public bool IsRotaryBackwLimit { get; set; }

        /// <summary>
        /// 旋转塔电机频率设定(Hz)
        /// </summary>
        public float RotaryFreqSet { get; set; }

        /// <summary>
        /// 旋转塔电机频率显示(Hz)
        /// </summary>
        public float RotaryFreq { get; set; }

        /// <summary>
        /// 旋转塔复位指示
        /// </summary>
        public bool IsRotaryOrg { get; set; }

        /// <summary>
        /// 旋转塔复位(┷）
        /// </summary>
        public bool RotaryOrg { get; set; }

        /// <summary>
        /// 旋转塔开启指示
        /// </summary>
        public bool IsRotaryOn { get; set; }

        /// <summary>
        /// 旋转塔开启(┷）
        /// </summary>
        public bool RotaryOn { get; set; }

        /// <summary>
        /// 故障复位
        /// </summary>
        public bool AlarmReset { get; set; }

        /// <summary>
        /// 进风变频故障报警
        /// </summary>
        public bool IsInletAirMotorError { get; set; }

        /// <summary>
        /// 出风变频故障报警
        /// </summary>
        public bool IsOutletAirMotorError { get; set; }

        /// <summary>
        /// 进风变频通讯报警
        /// </summary>
        public bool IsInletAirCommError { get; set; }

        /// <summary>
        /// 出风变频通讯报警
        /// </summary>
        public bool IsOutletAirCommError { get; set; }

        /// <summary>
        /// 外冷变频故障报警
        /// </summary>
        public bool IsExCoolMotorError { get; set; }

        /// <summary>
        /// 旋转变频故障报警
        /// </summary>
        public bool IsRotationMotorError { get; set; }

        /// <summary>
        /// 牵引变频故障报警
        /// </summary>
        public bool IsTractionMotorError { get; set; }

        /// <summary>
        /// 外冷变频通讯报警
        /// </summary>
        public bool IsExCoolCommError { get; set; }

        /// <summary>
        /// 旋转变频通讯报警
        /// </summary>
        public bool IsRotationCommError { get; set; }

        /// <summary>
        /// 牵引变频通讯报警
        /// </summary>
        public bool IsTractionCommError { get; set; }

        /// <summary>
        /// 风机电源报警
        /// </summary>
        public bool IsAirPowerError { get; set; }

        /// <summary>
        /// 急停开启报警
        /// </summary>
        public bool IsEmergencyStopError { get; set; }

        /// <summary>
        /// 旋转正转极限报警
        /// </summary>
        public bool IsRotationForwLimitError { get; set; }

        /// <summary>
        /// 旋转反转极限报警
        /// </summary>
        public bool IsRotationBackwLimitError { get; set; }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

    }
}
